/*
 * vi:ts=4:shiftwidth=4:expandtab
 *
 * iconvert.h  -  simple iconv() wrapper routine
 *
 * Copyright (C) 2002 by Zhang Le <ejoy@users.sourceforge.net>
 * Begin       : 02-Aug-2002
 * Last Change : 29-May-2005.
 *
 * This module is a wrapper for iconv() call. It makes converting between
 * unicode and other encodings an easy task. If your system does not have a
 * built-in iconv(), you need to install libiconv from: http://
 * www.gnu.org/software/libiconv/
 *
 * By default, each unicode char will be two bytes (UCS2). See unicode.h for
 * more.
 *
 * Typical usage:
 *    1. create a iconv_info object with desired encoding:
 *         iconv_info* ic = iconv_create("GBK");
 *         if (!ic) {
 *            error message goes here
 *         }
 *
 *    2. call iconv_from_unicode or iconv_to_unicode to convert strings
 *       between unicode and the encoding you choose:
 *        rc = iconv_from_unicode(ic, const uchar_t* from, size_t len_from,
 *             char* to, size_t len_to);
 *        rc = iconv_to_unicode(ic, const char* from, size_t
 *             len_from, uchar_t* to, size_t len_to);
 *
 *       all the lengths do not including null-terminators
 *       be sure to check the return values of those function calls, see below
 *       for more details
 *
 *    3. free the iconv_info object once you have your job done:
 *        rc = iconv_destory(ic);
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#ifndef _ICONVERT_H
#define _ICONVERT_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <iconv.h>

#include "unicode.h"

#if defined(__cplusplus)
extern "C" {
#endif

typedef struct {
    char *m_encoding;
    char       *m_buffer;
    iconv_t     m_iconv_from_unicode;
    iconv_t     m_iconv_to_unicode;
} iconv_info ;

/* utility functions */
iconv_info* iconv_create(const char* encoding);

/* Core conversion wrappers.
 * Input: 
 *       info:      the iconv_info object
 *       from:      the source buffer
 *       to:        the dest buffer (must have enough room)
 *       len_from:  length of the src buffer, not including null-terminator
 *       len_to:    length of the dest buffer
 *
 * The output buffer will be appended a null-terminitor.
 * The two conversion function return the size (>=0) of converted buffer (not
 * including the null-terminator), 
 * or -1 if failed to do conversion.
 * or -2 if output buffer overflow, 
 */
int iconv_from_unicode(iconv_info* info, const uchar_t* from, size_t len_from,
        char* to, size_t len_to);

int iconv_to_unicode(iconv_info* info, const char* from, size_t len_from,
        uchar_t* to, size_t len_to);

/* return 0 on success */
int iconv_destory(iconv_info* info);

uchar_t* str_to_unicode(const char* str, const char* encoding);

#if defined(__cplusplus)
}
#endif

#endif /* ifndef _ICONVERT_H */

